/**
 * 插件管理
 */
import { isObject, isMiddlewares, isFunction } from './checker';
import { IS_NATIVE_SYMBOL } from './symbol';

var combineMiddlewares = function combineMiddlewares(a, b) {
  return Object.keys(b).reduce(function (acc, timeline) {
    if (a[timeline]) {
      acc[timeline] = a[timeline].concat(b[timeline]);
    } else {
      acc[timeline] = Array.isArray(b[timeline]) ? b[timeline] : [b[timeline]];
    }

    return acc;
  }, a);
};
/**
 * [1, [2, 4]] ==> [1, 2, 4]
 */


function flat(enhancers) {
  return enhancers.filter(function (e) {
    return e;
  }).reduce(function (flatted, enhancer) {
    return flatted.concat(Array.isArray(enhancer) ? flat(enhancer) : enhancer);
  }, []);
}

var createStore = function createStore() {
  var $store = {};
  return {
    get: function get() {
      return $store;
    },
    set: function set(store) {
      $store = store;
    }
  };
};

var createPluginManager = function createPluginManager() {
  var plugins = {
    middlewares: {},
    props: []
  };

  var getPlugins = function getPlugins() {
    return plugins;
  };

  var contextStore = createStore();

  var use = function use() {
    for (var _len = arguments.length, rawEnhancers = new Array(_len), _key = 0; _key < _len; _key++) {
      rawEnhancers[_key] = arguments[_key];
    }

    var enhancers = flat(rawEnhancers) || []; // 插件能力聚合

    enhancers.forEach(function ($enhancer) {
      var isNative = !isObject($enhancer);
      var enhancer = isNative ? $enhancer(contextStore.get()) : $enhancer;
      Object.keys(enhancer).forEach(function (key) {
        if (isMiddlewares(key)) {
          var middlewares = enhancer[key];

          if (isObject(middlewares)) {
            plugins.middlewares = combineMiddlewares(plugins.middlewares, middlewares);
          }
        } else if (plugins[key]) {
          // native plugin 的 props 字段一定是一个函数
          if (isFunction(enhancer[key])) {
            // 先打一个标
            enhancer[key][IS_NATIVE_SYMBOL] = isNative;
          }

          plugins[key].push(enhancer[key]);
        }
      });
    });
  };

  var pureUse = function pureUse() {
    plugins.middlewares = {};
    plugins.props = [];

    for (var _len2 = arguments.length, rawEnhancers = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      rawEnhancers[_key2] = arguments[_key2];
    }

    use(rawEnhancers);
  };

  return {
    pureUse: pureUse,
    use: use,
    getPlugins: getPlugins,
    get: getPlugins,
    contextStore: contextStore,
    context: contextStore
  };
};

export default createPluginManager;